//
//  See the file 'LICENSE' for the license governing this code.
//

#include "Game.h"

Player::Player()
{
	wallJumpCount = 0;
	currentFoot = false;
	lastFootTime = 0.0;
	hangTime = 0.0f;
}

Player::~Player()
{
}

void Player::setProperties(Dictionary* dict)
{
	super::setProperties(dict);
	
	// give ourself a tag so we can easily be found with the getChildByTag() function
	this->setTag(kTagPlayer);
}

void Player::addSprite()
{
	sprite = new Sprite;
	
	// load the first standing sprite frame
	auto spriteFrame = this->profile->spriteFrameForKey("standing", kDirectionSouth);
	if( spriteFrame )
		sprite->initWithSpriteFrame(spriteFrame);
	
	this->addChild(sprite, 1);
}

void Player::update(float delta)
{
	super::update(delta);
	
	// calculate hang time
	if( hasContactBelow )
	{
		// landing noise
		if( hangTime > 0.25f )
			this->profile->playSound("land");

		hangTime = 0.0f;
	}
	else
		hangTime += delta;
	
	// make walking noises
	if( this->isAnimating(kActionWalk)
	&& !(this->isAnimating(kActionJump) || this->isAnimating(kActionRotate)) )
	{
		double currentTime = KITApp::getCurrentTimeInSeconds();
		if( currentTime - lastFootTime > 0.25f )
		{
			currentFoot = !currentFoot;
			lastFootTime = currentTime;
			this->profile->playSound(currentFoot ? "footL" : "footR");
		}
	}

	// handle touch input
	Point touchPos = this->getLevel().getTouchPosition();
	if( touchPos.x || touchPos.y )
		this->touch( delta,touchPos );

	// possibly stop and stand still
	else if( this->isAlive() )
	{
		b2Vec2 currentVelocity = body->GetLinearVelocity();
		if( fabsf(currentVelocity.x) < 0.25f
		&& fabsf(currentVelocity.y) < 0.25f
		&& !this->isAnimating(kActionJump) )
			this->standStill();
	}

}

void Player::touch(float delta, const Point& location)
{
	// if player is dead, return to life
	if( !this->isAlive() && this->sprite->getNumberOfRunningActions() == 0 )
	{
		this->health = this->maxHealth;
		this->sprite->setRotation(0);
		this->sprite->setPosition(Point::ZERO);
		return;
	}

	// gather some details
	float spriteHeight = this->sprite->getContentSize().height;
	Point playerScreenLocation = this->getPosition();
	Size winSize = Director::getInstance()->getWinSize();
	playerScreenLocation = playerScreenLocation - this->getLevel().getCameraPosition();
	
	b2Vec2 currentVelocity = body->GetLinearVelocity();
	b2Vec2 impulse(0.0f,0.0f);
	bool canJump = (fabsf(currentVelocity.y) <= 0.25f && (hasContactBelow || hasContactWithObject));
	bool canWallJump = false;//(hasContactToTheSide);
	
	float xVelocity = (location.x > playerScreenLocation.x ? 1.0f : -1.0f );
	float xDiff = fabsf(location.x - playerScreenLocation.x);
	if( xDiff < (spriteHeight / 2.0f) )
		xVelocity = 0.0f;
	if( xDiff > (spriteHeight * 2.0f) )
		xVelocity *= 2.0f;
	else if( xDiff > (spriteHeight * 1.25f) )
		xVelocity *= 1.5f;

	// jump
	//if( location.y > (winSize.height / 4.0f)
	if( location.y > (playerScreenLocation.y + (spriteHeight / 2.0f))
	&& (canJump || canWallJump) )
	{
		// make noise
		this->profile->playSound("jump");
		
		// apply an impulse
		float jumpXVelocity = xVelocity;
		if( jumpXVelocity )
			jumpXVelocity += (jumpXVelocity < 0.0f ? -3.0f : 3.0f);
		impulse.y = (1300.0f - (fabsf(jumpXVelocity) * 25.0f)) * delta;
		impulse.x = jumpXVelocity * 30.0f * delta;
		this->body->ApplyLinearImpulse(impulse, body->GetWorldCenter());

		// animate the flip
		this->animateFlip(xVelocity < 0.0f);
	}
	
	// walk
	else if( hasContactBelow && !this->isAnimating(kActionJump) && xVelocity )
	{
		// point our sprite in the right direction
		this->sprite->setFlippedX( xVelocity < 0.0f );
		
		// apply impulse
		if( fabsf(currentVelocity.x) < 5.0f )
		{
			impulse.y = 0.0f;
			impulse.x = xVelocity * 200.0f * delta;
			this->body->ApplyLinearImpulse(impulse, body->GetWorldCenter());
		}
		
		// animate
		if( !this->isAnimating(kActionWalk) )
		{
			this->sprite->setRotation(0);
			this->animate("walk", kActionWalk, false);
		}
	}
	
	// falling, so move slightly left or right
	else if( xVelocity )
	{
		// apply slight impulse
		impulse.y = 0.0f;
		impulse.x = xVelocity * 15.0f * delta;
		if( (currentVelocity.x >  5.0f && xVelocity > 0.0f)
		 || (currentVelocity.x < -5.0f && xVelocity < 0.0f) )
			impulse.x /= 4.0f;
		this->body->ApplyLinearImpulse(impulse, body->GetWorldCenter());
	}
}

void Player::beginContact(LevelObject* otherObject,b2Contact* contact)
{
	// search contact list for this object
	for( b2ContactEdge* edge = body->GetContactList(); edge; edge = edge->next )
	{
		if( otherObject == static_cast<LevelObject*>(edge->other->GetUserData()) )
		{
			// are we landing on top of this object?
			float heightFactor = this->sprite->getContentSize().height / Paralaxer::getPixelsPerMeter() / 4.0f;
			b2Vec2 othersPosition = edge->other->GetPosition();
			
			// if so, attack the object
			if( othersPosition.y < (body->GetPosition().y - heightFactor) )
				otherObject->attackedBy(this);
			
			// else, we are being attacked
			else
				this->attackedBy(otherObject);
		}
	}
}

void Player::attackedBy(LevelObject *o)
{
	Character* c = dynamic_cast<Character*>(o);
	Projectile* p = dynamic_cast<Projectile*>(o);
	
	// instant death
	if( c && c->isAlive() && c->isSolid() )
		this->die();
	else if( p )
		this->die();
}

void Player::standStill()
{
	// stop all actions, rotations, and translations
	this->sprite->stopAllActions();
	this->sprite->setRotation(0);
	this->sprite->setPosition(Point::ZERO);
	this->setStandingFrame();
}

void Player::setStandingFrame()
{
	// set sprite to first walking frame
	SpriteFrame* frame = spriteFrameFromAnimation(this->profile->animationForKey("jump"), 0);
	if( frame )
		this->sprite->setDisplayFrame(frame);
}

void Player::animateFlip(bool left)
{
	this->sprite->stopAllActions();
	this->animate("jump", kActionJump, false);

	// rotate in the air
	this->sprite->setRotation(0);
	RotateBy* p = new RotateBy();
	p->initWithDuration(0.89f, (left ? -360 : 360));
	p->setTag(kActionRotate);
	this->sprite->runAction(Sequence::create(
		p,
		CallFunc::create(CC_CALLBACK_0(self::setStandingFrame, this)),
		nullptr
		));
	p->release();
}

void Player::die()
{
	if( this->isAlive() )
	{
		health = 0;
		this->profile->playSound("die");
		this->standStill();
		
		// fall over
		this->sprite->runAction( RotateBy::create(0.2f, this->sprite->isFlippedX() ? 90 : -90) );
		this->sprite->runAction( MoveBy::create(0.3f, Point(this->sprite->isFlippedX() ? 25.0f : -25.0f, -20.0f)) );
		
		// complete the level with failure
		this->getLevel().completeLevel(false);
	}
}
